//
//  Fifo.m
//  MakeupMirror
//
//  Created by David Phillip Oster on 3/19/09.
//  Copyright 2009 David Phillip Oster. All rights reserved.
//

#import "Fifo.h"

// Consider the case of a Fifo length 1. 
// After adding an item one item, tail_ will be 0, and head_ will be 1. 
// Add a second item, and head_ will be 0, and tail_ will be 1.
// in both cases, the count is 1.


@implementation Fifo

- (id)initWithCapacity:(int)maxEntries {
  self = [super init];
  if (self) {
    container_ = [[NSMutableArray alloc] initWithCapacity:maxEntries];
    modulus_ = maxEntries+1; // see Note 1 above.
  }
  return self;
}


- (void)dealloc {
  [container_ release];
  container_ = nil;
  [super dealloc];
}

// head_ points at where the next item will be inserted.
// insert at the head.
// increments are modulo the maxcount.
- (void)addObject:(id)object {
  ++head_;
  if (modulus_ <= head_) {
    head_ = 0;
  }
  if (head_ == tail_) {
    // If the insert would make the container look full, increment the tail.
    ++tail_;
    if (modulus_ <= tail_) {
      tail_ = 0;
    }
  }
  if ([container_ count] < modulus_) {
    [container_ addObject:object];
  } else {
    [container_ replaceObjectAtIndex:head_ withObject:object];
  }
}

// or nil if there aren't any
// Add to head, remove from tail.
- (id)removeObject {
  id value = nil;
  if (head_ == tail_) {
    // empty. Done.
  } else {
    value = [container_ objectAtIndex:tail_];
    ++tail_;
    if (modulus_ <= tail_) {
      tail_ = 0;
    }
  }
  return value;
}

- (int)count {
  if (tail_ <= head_) {
    return head_ - tail_;
  } else {
    return modulus_ - (head_ - tail_);
  }
}


@end
